ColorGradingEditor = {}

local ColorGradingEditor_mt = Class(ColorGradingEditor)

function ColorGradingEditor.new(baseDir, settingsDir, customMt)
	local self = setmetatable({}, customMt or ColorGradingEditor_mt)

	addModEventListener(self)

	ColorGradingEditorUtil.appendedFunction(FSBaseMission, "registerActionEvents", self, "fsBaseMission_registerActionEvents")
	ColorGradingEditorUtil.overwrittenFunction(GuiOverlay, "loadOverlay", self, "guiOverlay_loadOverlay")

	self.uiFilename = Utils.getFilename("gui/ui_elements.png", baseDir)

	local colorGradingEditorItemsFrame = ColorGradingEditorItemsFrame.new(nil, nil)
	local colorGradingEditorSettingsFrame = ColorGradingEditorSettingsFrame.new(nil, nil)
	local colorGradingEditorScreen = ColorGradingEditorScreen.new(nil, nil, g_messageCenter, g_i18n, g_inputBinding, self, baseDir, settingsDir)
	local currentGui = FocusManager.currentGui

	Gui.CONFIGURATION_CLASS_MAPPING.cgeSlider = ColorGradingEditorSliderElement

	g_gui:loadProfiles(Utils.getFilename("gui/guiProfiles.xml", baseDir))
	g_gui:loadGui(Utils.getFilename("gui/ColorGradingEditorItemsFrame.xml", baseDir), "ColorGradingEditorItemsFrame", colorGradingEditorItemsFrame, true)
	g_gui:loadGui(Utils.getFilename("gui/ColorGradingEditorSettingsFrame.xml", baseDir), "ColorGradingEditorSettingsFrame", colorGradingEditorSettingsFrame, true)
	g_gui:loadGui(Utils.getFilename("gui/ColorGradingEditorScreen.xml", baseDir), "ColorGradingEditorScreen", colorGradingEditorScreen)

	if currentGui ~= nil then
		FocusManager:setGui(currentGui)
	end

	self.colorGradingEditorScreen = colorGradingEditorScreen

	return self
end

function ColorGradingEditor:fsBaseMission_registerActionEvents(fsBaseMission)
	local valid, eventId = fsBaseMission.inputManager:registerActionEvent(InputAction.TOGGLE_CGE, self, self.onToggleColorGradingEditor, false, true, false, true)

	fsBaseMission.inputManager:setActionEventTextVisibility(eventId, self.colorGradingEditorScreen.showHelp)
	self.eventId = eventId
end

function ColorGradingEditor:guiOverlay_loadOverlay(guiOverlay, superFunc, overlay, overlayName, imageSize, profile, xmlFile, key)
	local overlay = superFunc(guiOverlay, overlay, overlayName, imageSize, profile, xmlFile, key)

	if overlay ~= nil and overlay.filename == "g_cgeUIFilename" then
		overlay.filename = self.uiFilename
	end

	return overlay
end

function ColorGradingEditor:onToggleColorGradingEditor()
	g_gui:changeScreen(nil, ColorGradingEditorScreen)
end

function ColorGradingEditor:loadMap(filename)
	self.agsModLoaded = g_additionalSettingsManager ~= nil and g_additionalSettingsManager.addittionalSettings ~= nil and g_additionalSettingsManager.addittionalSettings.lighting ~= nil and g_currentMission.inGameMenu.pageAdditionalSettings ~= nil

	if self.agsModLoaded then
		self:agsModAddSupport()
	end

	self.colorGradingEditorScreen:onMapLoad()
end

function ColorGradingEditor:agsModGetCustomLighting(frame)
	if self.agsModLoaded then
		return g_additionalSettingsManager.addittionalSettings.lighting:getCurrentLighting()
	end
end

function ColorGradingEditor:agsModAddSupport()
	local frame = g_currentMission.inGameMenu.pageAdditionalSettings

	ColorGradingEditorUtil.appendedFunction(g_additionalSettingsManager.addittionalSettings.lighting, "onStateChange", self, "agsModLighting_onStateChange")
	ColorGradingEditorUtil.appendedFunction(frame, "onFrameClose", self, "agsModFrame_onFrameClose")

	local boxLayout = frame.boxLayout
	local multiLighting = frame.multiLighting
	local multiColorGrading = frame.multiLighting:clone(boxLayout)

	multiColorGrading.focusId = nil
	multiColorGrading.id = "multiColorGrading"
	multiColorGrading.elements[4]:setText(g_i18n:getText("ui_ags_preset_title"))
	multiColorGrading.elements[6]:setText(g_i18n:getText("ui_ags_preset_description"))

	function multiColorGrading.onClickCallback(_, state)
		local multiPreset = self.colorGradingEditorScreen.multiPreset

		multiPreset:raiseCallback("onClickCallback", state, multiColorGrading, true)
	end

	local currentGui = FocusManager.currentGui

	FocusManager:setGui("ingameMenuAdditionalSettings")
	FocusManager:loadElementFromCustomValues(multiColorGrading)
	FocusManager:setGui(currentGui)

	multiColorGrading.parent:removeElement(multiColorGrading)
	table.insert(boxLayout.elements, table.findListElementFirstIndex(boxLayout.elements, multiLighting, #boxLayout.elements) + 1, multiColorGrading)
	multiColorGrading.parent = boxLayout

	frame.multiColorGrading = multiColorGrading

	self.agsMultiColorGrading = multiColorGrading
end

function ColorGradingEditor:agsModLighting_onStateChange(lightingSetting, state, optionElement, loadFromSavegame)
	local customLighting = lightingSetting:getCurrentLighting()

	if customLighting ~= nil then
		self.colorGradingEditorScreen:updateDefaultPreset(customLighting.lighting.colorGradingDay, customLighting.lighting.colorGradingNight)
	else
		local baseLighting = g_currentMission.environment.baseLighting

		self.colorGradingEditorScreen:updateDefaultPreset(baseLighting.colorGradingDay, baseLighting.colorGradingNight)
	end
end

function ColorGradingEditor:agsModFrame_onFrameClose(frame)
	self.colorGradingEditorScreen:saveSettings()
end

ColorGradingEditor.new(g_currentModDirectory, g_currentModSettingsDirectory)